/* Copyright (C) 2007 Alejandro Álvarez
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */
 
#include <QContextMenuEvent>
#include "codeedit.h"

CodeEdit::CodeEdit(QWidget *parent, QString syntaxF): QTextEdit(parent),
						      syntax(this), contextMenu(this)
{
  if(!syntaxF.isEmpty())
    syntax.load(syntaxF);

  setUndoRedoEnabled(true);
  setTabStopWidth(32);
  setFrameStyle(QFrame::NoFrame);

  // ContextMenu
  connect(contextMenu.addAction(tr("Undo")), SIGNAL(triggered()),
	  this, SLOT(undo()));
  connect(contextMenu.addAction(tr("Redo")), SIGNAL(triggered()),
	  this, SLOT(redo()));

  contextMenu.addSeparator();

  connect(contextMenu.addAction(tr("Cut")), SIGNAL(triggered()),
	  this, SLOT(cut()));
  connect(contextMenu.addAction(tr("Copy")), SIGNAL(triggered()),
	  this, SLOT(cut()));
  connect(contextMenu.addAction(tr("Paste")), SIGNAL(triggered()),
	  this, SLOT(paste()));
  connect(contextMenu.addAction(tr("Delete")), SIGNAL(triggered()),
	  this, SLOT(deleteSelection()));

  contextMenu.addSeparator();

  connect(contextMenu.addAction(tr("Select all")), SIGNAL(triggered()),
	  this, SLOT(selectAll()));

  contextMenu.addSeparator();

  connect(contextMenu.addAction(tr("Toggle breakpoint")), SIGNAL(triggered()),
	  this, SLOT(toggleBreakpoint()));
  connect(this, SIGNAL( cursorPositionChanged() ), this, SLOT( braketsMatch() ) );
	  
  auto_indent=true;
  setAcceptDrops(false);
}

void CodeEdit::contextMenuEvent(QContextMenuEvent *e)
{
  setTextCursor(cursorForPosition(e->pos()));
  contextMenu.exec(e->globalPos());
}

void CodeEdit::undo()
{
  document()->undo();
}

void CodeEdit::redo()
{
  document()->redo();
}

void CodeEdit::deleteSelection()
{
  textCursor().removeSelectedText();
}

void CodeEdit::toggleBreakpoint()
{
  int line = 1;
  for(QTextBlock tb = document()->begin();
      tb.isValid();
      line++, tb = tb.next())
  {
    if(tb == textCursor().block())
    {
      emit toggleBreakpoint(line);
      return;
    }
  }
}

bool CodeEdit::event( QEvent * e )
{
	if( auto_indent && QEvent::KeyPress==e->type() )
	{
		QKeyEvent *k=(QKeyEvent *)e;
		if( Qt::Key_Return==k->key() ||  Qt::Key_Enter==k->key())
		{
			QTextCursor cursor=textCursor ();
			int pos=cursor.position();
			cursor.movePosition(QTextCursor::StartOfLine,QTextCursor::KeepAnchor);
			QString line=cursor.selectedText();
			QString start_blank;
			start_blank.append('\n');
			for(int i=0;i<line.length() && (line.at(i)==' ' || line.at(i)=='\t');i++)
				start_blank.append(line.at(i));
			cursor.setPosition(pos);
			cursor.insertText(start_blank);
			setTextCursor( cursor );
			return true;
		}
	}
	return QTextEdit::event(e);
}

void CodeEdit::setAutoindent(bool ai_ok)
{
	auto_indent=ai_ok;
}

bool CodeEdit::getAutoindent() {return auto_indent;}


void CodeEdit::braketsMatch()
{
	int pos=textCursor().position();
	if(pos>=0) syntax.braketsMacth(pos);
}



